JOB_DEFINITIONS = {
    "emtp": {
        "rid": "job-definition/cloudpss/emtp",
        "args": {
            "debug":
            "0",
            "n_cpu":
            "1",
            "solver":
            "0",
            "end_time":
            "1",
            "task_cmd":
            "",
            "step_time":
            "0.0001",
            "begin_time":
            "0",
            "task_queue":
            "",
            "initial_type":
            "0",
            "ramping_time":
            "0",
            "load_snapshot":
            "0",
            "save_snapshot":
            "0",
            "solver_option":
            "0",
            "output_channels": [{
                0: "aa",
                1: "1000",
                2: "compressed",
                3: "",
                4: [""]
            }],
            "max_initial_time":
            "1",
            "load_snapshot_name":
            "",
            "save_snapshot_name":
            "snapshot",
            "save_snapshot_time":
            "1"
        },
        "name": "电磁暂态仿真方案 1"
    },
    'sfemt': {
        "name": "移频电磁暂态仿真方案 1",
        "rid": "job-definition/cloudpss/sfemt",
        "args": {
            "begin_time":
            "0",
            "end_time":
            "1",
            "step_time":
            "0.0001",
            "output_channels": [{
                0: "aa",
                1: "1000",
                2: "compressed",
                3: "",
                4: [""]
            }],
            "solver":
            "0",
            "shift_freq":
            "50",
            "numerical_oscillation_suppression":
            "1",
            "ess":
            "0",
            "ess_time":
            "1e-6",
            "initial_type":
            "0",
            "ramping_time":
            "0",
            "max_initial_time":
            "1",
            "save_snapshot":
            "0",
            "save_snapshot_time":
            "1",
            "save_snapshot_name":
            "snapshot",
            "load_snapshot":
            "0",
            "load_snapshot_name":
            "",
            "task_queue":
            "",
            "solver_option":
            "0",
            "n_cpu":
            "1",
            "task_cmd":
            "",
            "debug":
            "0"
        }
    },
    "powerFlow": {
        "name": "潮流计算方案 1",
        "rid": "job-definition/cloudpss/power-flow",
        "args": {
            "UseBusVoltageAsInit": "1",
            "UseBusAngleAsInit": "1",
            "UseVoltageLimit": "1",
            "UseReactivePowerLimit": "1",
            "SkipPF": "0",
            "MaxIteration": "30"
        }
    },
    "iesLoadPrediction": {
        "name": "负荷预测方案 1",
        "rid": "job-definition/ies/ies-load-prediction",
        "arcs": {
            "startTime": "2022 -01-01 00:00:00",
            "endTime": "2022 -12-31 23:00:00",
            "layer_forcast": "0",
            "forcast_algorithm": "0",
            "ratio_sub": [],
            "ratio_feeder": [],
            "ratio_trans": []
        }
    },
    "iesPowerFlow": {
        "name": "时序潮流方案 1",
        "rid": "job-definition/ies/ies-power-flow",
        "arcs": {
            "startTime": "2022 -01-01 00:00:00",
            "endTime": "2022 -12-31 23:00:00",
            "solveMethod": "0",
            "maxIteration": "30"
        }
    },
    "iesEnergyStoragePlan": {
        "name": "储能规划方案 1",
        "rid": "job-definition/ies/ies-energy-storage-plan",
        "arcs": {
            "Planyear": "15",
            "NetConfig": [],
            "SourceConfig": [],
            "LoadConfig": []
        }
    }

}



import time
import sys
import os

sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))
import cloudpss
import time
import numpy as np
import pandas as pd
import json

if __name__ == '__main__':
    os.environ['CLOUDPSS_API_URL'] = 'http://10.101.10.45/'
    print('CLOUDPSS connected')
    cloudpss.setToken(
        'eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MSwidXNlcm5hbWUiOiJhZG1pbiIsInNjb3BlcyI6WyJtb2RlbDo5ODMzNSIsImZ1bmN0aW9uOjk4MzM1IiwiYXBwbGljYXRpb246OTgzMzUiXSwicm9sZXMiOlsiYWRtaW4iXSwidHlwZSI6ImFwcGx5IiwiZXhwIjoxNzI0NTU3MDIzLCJub3RlIjoiYSIsImlhdCI6MTY5MzQ1MzAyM30._Xuyo62ESKLcIAFeNdnfBM44yPiiXli9OPKvXDzL2rPV4J1_qsGZP--bsS1tXAVy-x8ooUIIAAG1yhwmZuk7-Q')
    print('Token done')
    # project = cloudpss.Model.fetch('model/admin/7744b02b-0636-5a39-8c16-eca939259ee1')
    topology = cloudpss.ModelTopology.fetch("-xrS3SewFhpVYKBtIXLk-XDLCQRQnUmlIbXS3s4sdPUkPKeAMhXHjRgZD1JPjPfQ","emtp",{'args':{}})
    
    
    
    # topology= project.fetchTopology(config={'args':{}})

    topology.dump(topology,'test.json')
    
    
    


import time
from cloudpss.runner.runner import HttpRunner, Runner
from cloudpss.runner.IESLabEvaluationResult import IESLabEvaluationResult
from ..utils import request
import json
from enum import IntEnum, unique

class IESLabEvaluationModel(object):
    _baseUri = 'api/ieslab-plan/rest/'
    _taskUri = 'api/ieslab-plan/taskmanager/getSimuLastTasks'
    _kindNameMap = {
        "投资组成": "investmentbanchandproportion",
        "资金来源": "capitalsource",
        "资产形式": "assetformation",
        "生产成本": "productioncost",
        "流动资金及财务费用": "workingcapitalandfinancialexpenses",
        "税率及附加": "projectcalculation",
    }

    # 财务评价基础参数接口默认值
    _financialParasDefaultValues = {
        "资产形式": {
            "fixedAssetsRatio": "95",
            "residualRrate": "5",
            "depreciationPeriod": "15",
            "reimbursementPeriod": "5"
        },
        "生产成本": {
            'annualSalary': "8",
            'capacity': "4",
            'insuranceRate': "0.25",
            'materialsExpenses': "5.0",
            'otherExpenses': "1.0",
            'welfareFactor': "0"
        },
        "流动资金及财务费用": {
            "annualAPCirculationTimes": "12",
            "annualARCirculationTimes": "12",
            "annualCashCirculationTimes": "12",
            "annualStockCirculationTimes": "12",
            "interestRateAndWorkingCapital": "4",
            "workingCapitalLoanRatio": "70"
        },
        "税率及附加": {
            "aleatoricAccumulationFundRate": "0",
            "basicDiscountRate": "8",
            "cityMaintenanceConstructionTaxTate": "5",
            "corporateIncomeTaxRate": "25",
            "educationFeePlus": "5",
            "electricityVATRate": "18",
            "fuelBoughtVATRate": "10",
            "hotColdVATRate": "12",
            "legalAccumulationFundRate": "10",
            "localEducationPlus": "2",
            "materialBoughtVATRate": "17",
            "steamSaleVATRate": "12"
        }
    }

    # 评价基础参数接口默认值
    _evaluationType = {
        "环保评价": "environmentalEvaluation",
        "能效评价": "energyEvaluation"
    }

    def __init__(self, simulationId):
        '''
            初始化
        '''
        self.simulationId = simulationId

    def _saveItemData(self, url, data):
        '''
            保存url链接对应的优化方案下财务评估模块的基础信息
            :param url string类型，表示优化方案的接口链接

            :return: dict 类型，代表方案对应的财务评价基础参数信息
        '''
        r = request('POST', url, data=json.dumps(data))
        dataList = json.loads(r.text)
        return dataList

    def _fetchItemData(self, url, planID):
        '''
            获取planID对应的优化方案下财务评估模块的基础信息
            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: dict 类型，为源数据的引用，代表方案对应的财务评价基础参数信息
        '''
        r = request('GET',
                    url,
                    params={
                        "simu_id": self.simulationId,
                        "planId": planID,
                    })
        data = json.loads(r.text)
        return data

    def GetFinancialParams(self, planID):
        '''
            获取planID对应的优化方案下财务评估模块的基础信息
            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: dict 类型，为源数据的引用，代表方案对应的财务评价基础参数信息
        '''
        dict_result = dict()
        for k, v in self._kindNameMap.items():
            kind = self._kindNameMap.get(k, k)
            url = self._baseUri + kind + '/'
            list = self._fetchItemData(url, planID)
            if (len(list['results']) == 0):
                data = {
                    "simu": self.simulationId,
                    "planId": planID,
                }
                if (k in self._financialParasDefaultValues):
                    data.update(self._financialParasDefaultValues[k])
                    res = self._saveItemData(url, data)
                    dict_result[v] = res
                else:
                    pass
            else:
                dict_result[v] = list['results']
        return dict_result

    def run(self, planID, type=None) -> Runner[IESLabEvaluationResult]:
        '''
            运行方案评估

            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间
            :params type:  string类型，任务类型：环保评价/能效评价

            :return: Runner[IESLabEvaluationResult]
        '''
        url = 'api/ieslab-plan/taskmanager/saveDataToclickhouse'
        CMD_TYPE = type if type is None else self._evaluationType[type]
        try:
            timeId = int(time.time() * 1000)
            r = request('GET',
                        url,
                        params={
                            "simuid": self.simulationId,
                            "planId": planID,
                            "CMD_TYPE": CMD_TYPE
                        })
            data = json.loads(r.text)
            return HttpRunner({'rid': 'function/ieslab/evaluation'},
                self.simulationId,
                timeId=timeId,
                planId=planID,
                cmdType=CMD_TYPE)
        except:
            raise Exception('方案评估开始计算失败')

    def EnvironmentalEvaluationRun(self, planID):
        '''
            运行环保评价方案评估

            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: 方案评估运行实例
        '''
        return self.run(planID, 'environmentalEvaluation')

    def EnergyEvaluationRun(self, planID):
        '''
            运行能效评价方案评估

            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: 方案评估运行实例
        '''
        return self.run(planID, 'energyEvaluation')

    def GetRunner(self, planID) -> Runner[IESLabEvaluationResult]:
        '''
            获得运行实例

            :return: Runner[IESLabEvaluationResult]
        '''
        return HttpRunner({'rid': 'function/ieslab/evaluation'},
            self.simulationId,
            planId=planID)


@unique
class OptimizationMode(IntEnum):
    经济性 = 0
    环保性 = 1


from cloudpss.runner.IESLabPlanResult import IESLabPlanResult
from cloudpss.runner.runner import HttpRunner, Runner
from ..utils import request, fileLoad
import json
from enum import IntEnum, unique


class IESLabPlanModel(object):
    _baseUri = 'api/ieslab-plan/taskmanager/getSimuLastTasks'

    def __init__(self, simulationId):
        '''
            初始化
        '''
        self.simulationId = simulationId
        self.optimizationInfo = self.GetOptimizationInfo()
        # self.OptimizationMode = OptimizationMode

    def _fetchItemData(self, url, params):
        '''
            获取当前算例的优化目标设置信息

            :return: List 类型，包括优化目标和全局参数储能动作灵敏度，若没有设置则返回 []
        '''
        r = request('GET', url, params=params)
        data = json.loads(r.text)
        return data['results']

    def GetOptimizationInfo(self):
        '''
            获取当前算例的优化目标设置信息

            :return: Dict 类型，例如：{'OptimizationMode': <OptimizationMode.经济性: 0>, 'StoSen': 0.1}
        '''
        try:
            url = f'api/ieslab-plan/rest/simuOpt/'
            params = {"simu_id": self.simulationId}
            r = self._fetchItemData(url, params)
            if (len(r) == 0): 
                return {
                    "OptimizationMode": OptimizationMode['经济性'],
                    "StoSen": 0.1
                }
            else:
                value = json.loads(r[0]['opt_params'])
                return {
                    "OptimizationMode": OptimizationMode(value['OptimizationMode']),
                    "StoSen": value['StoSen']
                }
        except:
            raise Exception('获得优化目标设置失败')

    def SetOptimizationInfo(self, data: dict):
        '''
            设置当前算例的优化目标

            :param data: dict 类型，例如：{'OptimizationMode': <OptimizationMode.经济性: 0>, 'StoSen': 0.1}

            :return: boolean 类型，为 True 则设置成功
        '''
        try:
            url = f'api/ieslab-plan/rest/simuOpt/'
            params = {"simu_id": self.simulationId}
            r = self._fetchItemData(url, params)
            opt_params = {
                "OptimizationMode": data.get('OptimizationMode', '').value,
                "StoSen": data.get('StoSen', ''),
                "ProjectPeriod": "20"
            }
            if(len(r) == 0):
                payload = {
                    "simu_id": self.simulationId,
                    "opt_params": json.dumps(opt_params)
                }
                r = request('POST',
                            url,
                            data=json.dumps(payload))
                return True
            else:
                url2 = f'api/ieslab-plan/rest/simuOpt/{r[0]["id"]}/'
                payload = {
                    "simu_id": self.simulationId,
                    "opt_params": json.dumps(opt_params),
                    "id": r[0]["id"]
                }
                r = request('PUT',
                            url2,
                            data=json.dumps(payload))
                return True
        except:
            return False
    
    def run(self) -> Runner[IESLabPlanResult]:
        '''
            生成方案优选算例

            :return: Runner[IESLabPlanResult]
        '''
        isRunning = self.GetLastTaskResult()
        if isRunning:
            raise Exception('该算例正在运行！请从浏览器算例页面点击结束运行或者调用IESPlan对象的kill接口终止计算后重试！')
        else:
            url = 'api/ieslab-plan/taskmanager/runOptimization'
            opt = {
                "OptimizationMode": self.optimizationInfo.get('OptimizationMode', 0).value,
                "ProjectPeriod": "20",
                "StoSen": self.optimizationInfo.get('StoSen', 0.1)
            }
            try:
                r = request('GET',
                            url,
                            params={
                                "simuid":
                                self.simulationId,
                                "optPara":
                                json.dumps(opt)
                            })
                data = json.loads(r.text)
                return HttpRunner({}, self.simulationId)
            except:
                raise Exception('生成方案优选算例失败')

    def GetRunner(self) -> Runner[IESLabPlanResult]:
        '''
            获得运行实例

            :return: Runner[IESLabPlanResult]
        '''
        return HttpRunner({}, self.simulationId)
    
    def kill(self) -> bool:
        '''
            停止并删除当前运行的优化算例
        '''
        res = IESLabPlanResult(self.simulationId).getLastTaskResult()
        error = res.get('error', 0)
        if error == 0:
            data = res.get('data', {})
            if data is not None:
                taskID = data.get('task_id', '')
        url = f'api/ieslab-plan/taskmanager/removeOptimizationTask'
        try:
            r = request('GET',
                        url,
                        params={
                            "taskid": taskID,
                            "stopFlag": '2'
                        })
            json.loads(r.text)
            return True
        except:
            return False

    def GetLastTaskResult(self)-> bool:
        '''
            获取最后一次运行的taskID的运行结果与日志

            :return: boolean 类型
        '''
        isRunning = True
        res = IESLabPlanResult(self.simulationId).getLastTaskResult()
        error = res.get('error', 0)
        if error == 0:
            data = res.get('data', {})
            if data is not None:
                status = data.get('status', '')
                if status == 'stop':
                    isRunning = False
        logs = IESLabPlanResult(self.simulationId).GetLogs()
        if logs is not None:
            for log in logs:
                if(log.get('data', '') == 'run ends'):
                    isRunning = False
                    break
        return isRunning


# @unique
class OptimizationMode(IntEnum):
    经济性 = 0
    环保性 = 1


from .result import Result
import re
import copy
class IESLabTypicalDayResult(Result):
    def __init__(self, *args, **kwargs):
        """
            初始化
        """
        Result.__init__(self, *args, **kwargs)
        self.__plotIndex = 0
        self.__typicalIndex = 0
        self.__type_list =['总辐射','散射辐射', '直射辐射','天顶角', '环境温度', '湿球温度','土壤温度', '10m风速', '50m风速','电负荷', '热负荷','冷负荷','氢负荷']
        self.__map_load = { 'maxElectricalLoad':'电负荷', 'maxHeatLoad':'热负荷','maxCoolLoad':'冷负荷','maxHydrogenLoad':'氢负荷' }
        self.result = {'TypicalMonth': [{'月份': int,'持续天数': [],**{key: [] for key in self.__type_list}} for i in range(12)],'TypicalDay': []}

    def __readPlotResult(self):
        length = self.db.getMessageLength()
        if (length > self.__plotIndex):
            for num in range(self.__plotIndex, length):# update TypicalMonth
                val = self.db.getMessage(num)
                if val['type'] == 'plot':# 每个月各类型数据的各个典型日的数据，由于部分月份可能没有电冷热负荷，某月的某个典型日可能缺少冷热负荷
                    key_re = re.split('-month',val['key'])#格式为：散射辐射-month1，re后分别为类型和月份
                    typical_month_m = self.result['TypicalMonth'][int(key_re[1])-1]
                    typical_month_m['月份'] = int(key_re[1])
                    val_data_traces = val['data']['traces']
                    #val['data']['traces'][i]['name']格式为：典型日1-共31天，re正则后[0]为典型日顺序，[1]为持续天数
                    typicalNum = int(re.findall('\d+',val_data_traces[-1]['name'])[0])
                    for i in range(typicalNum):#取该类型的最后一个典型日顺序，当该类型缺少后排典型日时，小于实际典型日数量
                        typical_month_m[key_re[0]].append([])
                        if key_re[0]  == '环境温度':#各类版本气象数据均有环境温度数据，其典型日数量为实际数量
                            typical_month_m['持续天数'].append(int(re.findall('\d+',val_data_traces[i]['name'])[1]))
                    # 当前排典型日缺少数据时，该类型数据为空list[]；当后排典型日缺少数据时，该类型数据为空
                    for i in range(len(val_data_traces)):
                        typical_month_m[key_re[0]][int(re.findall('\d+',val_data_traces[i]['name'])[0])-1] = copy.deepcopy(val_data_traces[i]['y'])            
            
            # update TypicalDay based on TypicalMonth
            for m in range(12):
                typical_month_m = self.result['TypicalMonth'][m]
                typical_month_m_day = len(typical_month_m['持续天数'])
                for i in range(typical_month_m_day):
                    self.result['TypicalDay'].append({'info':{'typicalDayID': int, 'name': str, 'duration': int, **{key: 0.0 for key in self.__map_load}},
                    'data': {**{key: [] for key in self.__type_list}}})
                    typical_day_index = self.result['TypicalDay'][self.__typicalIndex]
                    typical_day_index['info']['typicalDayID'] = self.__typicalIndex
                    typical_day_index['info']['name'] = str(m+1) + '月典型日' + str(i+1)
                    typical_day_index['info']['duration'] = typical_month_m['持续天数'][i]
                    for key,value in self.__map_load.items():
                        # 分别处理该典型日无此类型负荷数据，缺少后序典型日，缺少前序典型日的情况
                        if typical_month_m.get(value) and i <len(typical_month_m[value]) and len(typical_month_m[value][i]):
                            typical_day_index['info'][key] = max(typical_month_m[value][i])
                    for type_i in self.__type_list:
                        # 某月冷热负荷可能缺少后续典型日数据
                        if typical_month_m[type_i] and i < len(typical_month_m[type_i]):
                            typical_day_index['data'][type_i] = typical_month_m[type_i][i]
                    self.__typicalIndex += 1
            self.__plotIndex = length
            
    def GetTypical(self):
        '''
            获取所有的 GetTypical 典型日数据

            >>> result.GetTypical()
            {...}
        '''
        self.__readPlotResult()
        return self.result['TypicalDay']
        

    def GetTypicalDayNum(self):
        '''
            获取当前result的典型日总数
            
            :return: int类型，代表典型日数量
        '''
        self.__readPlotResult()
        return self.__typicalIndex
        
    def GetTypicalDayInfo(self,dayID):
        '''
            获取dayID对应典型日的基础信息
            
            :params: dayID int类型，表示典型日的ID，数值位于 [0，典型日总数-1] 中
            
            :return: dict类型，代表典型日的基础信息，包括典型日所代表的日期范围、典型日的名称等
        '''
        self.__readPlotResult()
        try:
            return self.result['TypicalDay'][dayID]['info']
        except:
            raise Exception('未查询到该数据')
        

    def GetTypicalDayCurve(self,dayID, dataType):
        '''
            获取dayID对应典型日下dataType参数的时序曲线
            
            :params: dayID int类型，表示典型日的ID，数值位于 [0，典型日总数-1] 中
            :params: dataType enum类型，标识典型场景的数据类型
            有总辐射、直射辐照、散射辐照、环境温度、湿球温度、土壤温度、10m风速、50m风速、电负荷、热负荷、冷负荷和氢负荷等类型，以该典型场景实际包含的类型为准
            
            :return: list<float>类型，代表以1h为时间间隔的该参数的日内时序曲线
        '''
        self.__readPlotResult()
        try:
            return self.result['TypicalDay'][dayID]['data'][dataType]
        except:
            raise Exception('未查询到该数据')
        
        
        
    def GetTypicalMonth(self):
        '''
            获取所有的 GetTypicalMonth 数据
            
            >>> result.GetTypicalMonth()
            
            :return: list<dict>类型，代表各月各类型的典型日数据
        '''
        self.__readPlotResult()
        return self.result['TypicalMonth']
    
    def GetTypicalMonthID(self,monthID):
        '''
            获取第monthID月各类型的典型日数据

            >>> result.GetTypicalMonthID()
            
            :params: monthID int类型，表示典型月的ID，数值位于 1-12 之间

            :return: dict类型，代表第monthID月各类型的典型日数据
            {...}
        '''
        self.__readPlotResult()
        try:
            return self.result['TypicalMonth'][monthID-1]
        except:
            raise Exception('未查询到该数据')
        
    
    def GetTypicalMonthCurve(self,monthID, dataType):
        '''
            获取monthID对应典型日下dataType参数的时序曲线
            
            :params: monthID int类型，表示典型月的ID，数值位于 1-12 之间
            :params: dataType enum类型，标识典型场景的数据类型
            有总辐射、直射辐照、散射辐照、环境温度、湿球温度、土壤温度、10m风速、50m风速、电负荷、热负荷、冷负荷和氢负荷等类型，以该典型场景实际包含的类型为准
            
            :return: list<list>类型，代表以1h为时间间隔的该参数的典型日内时序曲线
        '''
        self.__readPlotResult()
        try:
            return self.result['TypicalMonth'][monthID-1][dataType]
        except:
            raise Exception('未查询到该数据')        


class Component(object):
    """
        元件类

        实例变量说明：

        definition 元件定义， 连接线没有definition

        args 元件参数数据，连接线没有参数数据

        pins 元件引脚数据，连接线没有引脚数据

        shapes diagram-component 表示元件，diagram-edge 表示连接线

    """

    def __init__(self, diagram: dict = {}):
        self.__dict__.update(diagram)

    def __getitem__(self, attr):
        return super(Component, self).__getattribute__(attr)

    def toJSON(self):
        """
            类对象序列化为 dict
            :return: dict

            >>> comp.toJSON()
        """
        cells = {**self.__dict__}

        return cells



import os, sys

sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))

import cloudpss
if __name__ == '__main__':
    job = cloudpss.currentJob()
    
    
    job.plot(traces= [
        { 'name': 'trace1', 'type': 'scatter', 'x': [1, 2, 3], 'y': [1, 2, 3] },
            { 'name': 'trace2', 'type': 'bar', 'x': [1, 2, 3], 'y': [1, 2, 3] },
        ], layout= {
            'xaxis': { 'title': 'x坐标' },
            'yaxis': { 'title': 'y坐标' },
        },title= 'My Plot', key='plot-1')


import numpy as np

def loadPyData(data): 

    """
        matlab处理python数据中的list数据.

        :data: 数据

        :return: 用户信息 

        >>> matlab.loadPyData(data)
        {
            ...
        }
    """

    if isinstance(data, dict): 
        tmp={}
        for key,val in data.items():
            tmp[key]=loadPyData(val)
        return tmp
    if isinstance(data, list): 
        try:
            x=np.array(data)
            if x.dtype.type==np.str_:
                return [loadPyData(val) for val in  data]
            if x.dtype.type==np.object_:
                return [loadPyData(val) for val in  data]
            return memoryview(x)
        except Exception as e:
            return [loadPyData(val) for val in  data]
    else:
        return data


from .topology import ModelTopology
from .implements import ModelImplement
from ..utils import request, graphql_request
from ..runner import Runner


class ModelRevision(object):
    """
        表示一个项目的版本数据

        实例变量说明：

        implements          项目当前版本的实现数据

        parameters          项目当前版本的参数结果

        pins                项目当前版本的引脚信息

        documentation       项目当前版本的文档信息



    """
    def __init__(self, revision: dict = {}):
        """
            初始化
        """
        for k, v in revision.items():
            if k == 'implements':
                self.__dict__[k] = ModelImplement(v)
            else:
                self.__dict__[k] = v

    def __getitem__(self, attr):
        return super(ModelRevision, self).__getattribute__(attr)

    def toJSON(self):
        """
            类对象序列化为 dict
            :return: dict
        """

        revision = {**self.__dict__, 'implements': self.implements.toJSON()}
        return revision

    def getImplements(self):
        """
            获取当前版本的实现

            :return: 实现实例

            >>> revision.getImplements()
        """

        return self.implements

    def run(self, job, config, name=None, rid='', **kwargs):
        """
            运行某个指定版本的项目

            :params job:  调用仿真时使用的计算方案，为空时使用项目的第一个计算方案
            :params config:  调用仿真时使用的参数方案，为空时使用项目的第一个参数方案
            :params name:  任务名称，为空时使用项目的参数方案名称和计算方案名称
            :params rid:  项目rid，可为空

            :return: 返回一个运行实例

            >>> revision.run(revision,job,config,'')
        """
        revision = ModelRevision.create(self)
        return Runner.create(revision['hash'], job, config, name, rid,
                             **kwargs)

    @staticmethod
    def create(revision, parentHash=None):
        """
            创建一个新版本

            :params: revision 版本数据

            :return: 项目版本hash

            >>> ModelRevision.create(model.revision)
            {hash:'4043acbddb9ce0c6174be65573c0380415bc48186c74a459f88865313743230c'}
        """

        query = """
                    mutation($a:CreateModelRevisionInput!){createModelRevision(input:$a){
                            hash
                    }}
                """
        r = revision.toJSON()
        del r['hash']
        variables = {'a': {**r, 'parent': parentHash}}

        r = graphql_request(query, variables)
        if 'errors' in r:
            raise Exception(r['errors'])
        return r['data']['createModelRevision']

    def fetchTopology(self, implementType, config, maximumDepth):
        """
            获取当前项目版本的拓扑数据

            :params implementType:  实现类型
            :params config:  项目参数
            :params maximumDepth:  最大递归深度，用于自定义项目中使用 diagram 实现元件展开情况

            :return:  一个拓扑实例

            >>> topology=revision.fetchTopology()
                topology=revision.fetchTopology(implementType='powerFlow',config=config) # 获取潮流实现的拓扑数据
                topology=revision.fetchTopology(maximumDepth=2) # 获取仅展开 2 层的拓扑数据

        """

        if self.hash is not None:
            return ModelTopology.fetch(self.hash, implementType, config,
                                       maximumDepth)
        return None



import os,sys
import time
import uuid
sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))
import cloudpss 
import logging

def next(m):
    m.write({'type': 'debug', 'step': '-1'})
def goto(m,step):
    m.write({'type': 'debug', 'step': step})
def pause(m):
    m.write({'type': 'debug', 'step': 10})


def memory(m,path,buffer,offset):
    m.write({'type': 'memory', 'path': path,'value':buffer,'offset':offset})
    

    
if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG)
    os.environ['CLOUDPSS_API_URL'] = 'http://10.101.10.45/'
    cloudpss.setToken('eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MSwidXNlcm5hbWUiOiJhZG1pbiIsInNjb3BlcyI6WyJtb2RlbDo5ODMzNSIsImZ1bmN0aW9uOjk4MzM1IiwiYXBwbGljYXRpb246OTgzMzUiXSwicm9sZXMiOlsiYWRtaW4iXSwidHlwZSI6ImFwcGx5IiwiZXhwIjoxNzI2MjA4NjM5LCJub3RlIjoidGVzdDIiLCJpYXQiOjE2OTUxMDQ2Mzl9.ZlFSPCH3u4OSBlDBJWve4GZkWIZAku_DI1j-or-uDtAtyDy-RZecn7RBhymsILNPsmKnwkXcx1UnsjRNiAf5yA')
    # job = cloudpss.Job.fetch()
    
    model = cloudpss.Model.fetch('model/admin/debug')
    
    jobConfig = model.jobs[4]
    jobConfig['args']['stop_on_entry']='1'
    modelJob =model.run(job=jobConfig)
    
    view =modelJob.result
    i=0
    while not view.end:
        i+=1
        print(view.getLogs())
        # print(view.getPlots())
        print('status2',view.end)
        view.next()
        view.control({
            "key":"/component_new_constant_1",
            "value":str(i),
            'uuid': str(uuid.uuid1()),
        })
        time.sleep(0.01)


from .diagram import DiagramImplement


class ModelImplement(object):
    """
        实现类
    """
    def __init__(self, implements: dict = {}):
        """
            初始化
        """
        for k, v in implements.items():
            if k == 'diagram':
                self.__dict__[k] = DiagramImplement(v)
            else:
                self.__dict__[k] = v

    def __getitem__(self, attr):
        return super(ModelImplement, self).__getattribute__(attr)

    def toJSON(self):
        """
            类对象序列化为 dict
            :return: dict

             >>> implement.toJSON()
        """
        implements = {**self.__dict__, 'diagram': self.diagram.toJSON()}
        return implements

    def getDiagram(self):
        """
            获取拓扑实现，不存在返回空

            :return: 示意图实例

            >>> implement.getDiagram()
        """

        return getattr(self, 'diagram', None)



import json
from ..utils import request, graphql_request


class ModelTopology():
    """
        项目拓扑类

        通过 该类的静态方法 fetch 获取一个拓扑实例

        实例变量说明：

        components 摊平后的拓扑元件，参数和引脚不再保留表达式的形式，如果元件为拓扑实现，并有读取权限时将被展开

        mappings   拓扑分析后的一些映射数据


    """
    __All__ = ['ModelTopology']

    def __init__(self, topology: dict = {}):
        self.__dict__.update(topology)

    def toJSON(self):
        """
            将类转换为 dict 数据
        """
        data = {**self.__dict__}

        return data

    @staticmethod
    def dump(topology, filePath, indent=None):
        """
            以 JSON 格式保存数据到指定文件

            :params: topology 拓扑实例
            :params: file 文件路径
            :params: indent json 格式缩进
        """
        data = topology.toJSON()
        f = open(filePath, 'w', encoding='utf-8')
        json.dump(data, f, indent=indent)
        f.close()

    @staticmethod
    def fetch(hash, implementType, config, maximumDepth=None):
        """
            获取拓扑

            :params: hash 
            :params: implementType 实现类型
            :params: config 参数方案
            :params: maximumDepth 最大递归深度，用于自定义项目中使用 diagram 实现元件展开情况

            : return: 拓扑实例

            >>> data = ModelTopology.fetch('','emtp',{})

        """
        args = {} if config is None else config['args']
        query = """
            query($a:ModelTopologyInput!){
            modelTopology(input:$a){
                components
                mappings
            }}
        """

        variables = {
            "a": {
                'hash': hash,
                'args': args,
                'acceptImplementType': implementType,
                'maximumDepth': maximumDepth
            }
        }
        data = graphql_request(query, variables)
        if 'errors' in data:
            raise Exception(data['errors'][0]['message'])

        return ModelTopology(data['data']['modelTopology'])



from .matlab import loadPyData
from .httprequests import request
from .yamlLoader import fileLoad
from .dataEncoder import MatlabDataEncoder, DateTimeEncode
from .graphqlUtil import graphql_request

__all__ = ['request', 'fileLoad', 'MatlabDataEncoder', 'DateTimeEncode', 'graphql_request']



import os,sys
import time
sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))
import cloudpss 
import logging


    
if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG)
    os.environ['CLOUDPSS_API_URL'] = 'http://10.101.10.45/'
    cloudpss.setToken('eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MSwidXNlcm5hbWUiOiJhZG1pbiIsInNjb3BlcyI6WyJtb2RlbDo5ODMzNSIsImZ1bmN0aW9uOjk4MzM1IiwiYXBwbGljYXRpb246OTgzMzUiXSwicm9sZXMiOlsiYWRtaW4iXSwidHlwZSI6ImFwcGx5IiwiZXhwIjoxNzI2MjA4NjM5LCJub3RlIjoidGVzdDIiLCJpYXQiOjE2OTUxMDQ2Mzl9.ZlFSPCH3u4OSBlDBJWve4GZkWIZAku_DI1j-or-uDtAtyDy-RZecn7RBhymsILNPsmKnwkXcx1UnsjRNiAf5yA')
    # job = cloudpss.Job.fetch()
    
    model = cloudpss.Model.fetch('model/admin/shm_test_2')
    
    jobConfig = model.jobs[0]
    jobConfig['args']['stop_on_entry']='1'
    runner =model.run(job=jobConfig)
    
  
    ## 程序启动时将在第一个时步暂停，以下代码实现每隔 5s 算例往前运行 0.5s
    step=1
    i=1
    t=0
    while not runner.status():
        time.sleep(0.1)
        runner.result.goto(t)
        i+=1
        runner.result.writeShm('flag2',float(i),0)
        # d =runner.result.getPlotChannelData(1,'a:0')
        # runner.result.writeShm('flag2',float(i),0)
        # i+=1
        # if t ==0:
        #     t+=step
            
        #     runner.result.goto(t)
        # if d is not None:
        #     ## 最后一个时间点为小于停止时间点的前一个时步，所以需要计算每个时步差（当前算例为等步长计算），由于输出的时间点为浮点数，因此需要考虑精度问题，这里使用了最后三个时步的差值来判断是否到达停止时间点
        #     if d['x'][-1]+(d['x'][-1]-d['x'][-3]) >=t:
        #         t+=step
        #         runner.result.goto(t)
        #         # runner.result.writeShm('flag2',float(t),0)
                
        #         time.sleep(5)
        #     print(d['x'][-1],d['x'][-3])
    print("end")



from .receiver import Receiver
from .result import PowerFlowResult, EMTResult, Result
from .storage import Storage
from .runner import Runner

__all__ = ['Runner', 'Result', 'PowerFlowResult',
           'EMTResult', 'Receiver', 'Storage']



import os, sys

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))
import cloudpss
import json
import time
import matplotlib.pyplot as plt
if __name__ == '__main__':
    #cloudpss.setToken('eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MTEzLCJ1c2VybmFtZSI6ImRwc2hzdyIsInNjb3BlcyI6W10sInR5cGUiOiJhcHBseSIsImV4cCI6MTY4ODUyMzQ4NSwiaWF0IjoxNjgwNzQ3NDg1fQ.1c_ZhFDsx-Agn9cxr781UVr5OtB7LaJX96_I4zG0NvbJeuxyGtDPAe4RReWbbzpAZOzqMtCUCUuzl5YrOA5Daw')


    ### 获取指定 rid 的项目
    os.environ['CLOUDPSS_API_URL'] = 'http://166.111.60.76:50001/'
    cloudpss.setToken('eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6NjExLCJ1c2VybmFtZSI6ImRwc2hzdyIsInNjb3BlcyI6W10sInR5cGUiOiJhcHBseSIsImV4cCI6MTY4OTU1NzgxNCwiaWF0IjoxNjgxNzgxODE0fQ.4zYPiEDRK9nm8zRlCUiquPjp0RvP910Hxg8OO7ahFZIcre-lW_4eo-vYjVRZ40EiIo8VxH9lR9TEo-ZZ4Fck4g')
    model = cloudpss.Model.fetch('model/dpshsw/IEEE39')

    try:
        job = model.jobs[1]
        job['args']['end_time'] = 0.06
        runner = model.run(config={
            "args": {
                "save_snapshot_new": "1",  ## 是否保存快照，1 保存，0 不保存
                "save_snapshot_time_new": "0.05",  ## 保存快照的时间  单位：秒
                "snapshot_number_new": "12312",  ## 快照编号，用于区分不同的快照
            }
        })
        while  not runner.status() :
            print('running',flush=True)
            #print(runner.result.getPlots(0))
            time.sleep(1)

        print('end')

    except Exception as e:
        print('error',e)

    print(runner.result.getPlotChannelData(0, "#KL4:0"))

    #comp = model.getComponentByKey('/component_new_breaker_3_p_98:0')
    comp = model.getComponentByKey("component_new_constant_118")
    comp.args["Value"] = "0"


    print('restart from snapshot')

    job = model.jobs[1]
    job['args']['begin_time'] = 0.05
    job['args']['end_time'] = 0.10
    runner = model.run(config={
        "args": {"load_snapshot_new": "1",  ## 是否加载快照，1 加载，0 不加载
                 "load_snapshot_time_new": "0.05",  # 加载快照的时间  单位：秒
                 "load_snapshot_number_new": "12312"  # 快照编号，用于区分不同的快照
                 }
    })
    plotMap = {}
    fig, ax = plt.subplots()
    while not runner.status():
        time.sleep(1)
        print('running', flush=True)
        print(runner.result.getLogs())
        plotKeys = runner.result.getPlotChannelNames(1)
        if plotKeys is not None:
            for val in plotKeys:
                line = plotMap.get(val, None)

                # 获取曲线数据
                channel = runner.result.getPlotChannelData(1, val)
                if line is None:

                    line, = ax.plot(channel['x'], channel['y'], label=val)
                    plotMap[val] = line
                else:
                    line.set_data(channel['x'], channel['y'])

        plt.pause(0.1)
    plt.show()
    print(runner.result.getPlotChannelData(0, "#KL4:0"))


import os

from .utils import graphql_request


def setToken(token):
    """
        设置 用户申请的 sdk token 

        :params: token token 

        >>> cloudpss.setToken(token)
    """
    os.environ['CLOUDPSS_TOKEN'] = token


def userToken(token):
    try:
        
        query = '''
            query ($input: AccountTokenInput!) {               
                accountToken(input: $input){                   
                    user{                      
                        name                 
                    }               
                } 
            }          
        '''

        result = graphql_request(query, {'input': {'token': token}})
    except Exception as e:
        query = '''
            query ($input: UserTokenInput!) {               
                accountToken:userToken(input: $input){                   
                    user{                      
                        name                 
                    }               
                } 
            }          
        '''

        result = graphql_request(query, {'input': {'token': token}})
    if 'errors' in result:
        raise Exception(result['errors'])
    return result['data']['accountToken']['user']


def userName(token=None):
    if token is None:
        token = os.environ.get('CLOUDPSS_TOKEN', None)
    if token is None:
        raise Exception('token is None')
    return userToken(token)['name']


def verifyToken(token):
    return userToken(token)




from aiohttp import ClientSession, WSMsgType
import asyncio
import time
# ws://10.101.10.45/api/streams/token/fe943635-0dc9-40c8-8a27-39d02af66894

    
class WebsocketClient():
    message = []
    _status=0
    # __init__(self):
    def on_closed(self):
        self._status = 1
        
        print("close")
    def on_error(self,msg):
        self._status = 1
        print("error")
    def on_open(self):
        self._status = 0
        print("open")
    # websocket
    def __iter__(self):
        return self

    def __next__(self):
        maxLength = len(self.messages)
        if self.index < maxLength:
            message = self.messages[self.index]
            self.index += 1
            return message
        raise StopIteration()
    
    def __aiter__(self):
        return self
    
    async def receive(self):
        msg = await self.ws.receive()
        if msg.type == WSMsgType.BINARY:
            self.message.append(msg)
        if msg.type == WSMsgType.TEXT:
            self.message.append(msg)
        elif msg.type == WSMsgType.CLOSE:
            print(msg)
        elif msg.type == WSMsgType.CLOSED:
            print(msg)
            self.on_closed()
            await self.session.close()
        elif msg.type == WSMsgType.ERROR:
            self.on_error(msg.data)
        return msg
    async def __anext__(self):
        msg = await self.receive()
        if msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING, WSMsgType.CLOSED):
            raise StopAsyncIteration
        return msg
        # msg = await self.receive()
        # if msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING, WSMsgType.CLOSED):
        #     raise StopAsyncIteration
        # return msg 
    
    async def websocket_connect(self,url):
        
        # session = await ClientSession()
        # ws = await session.ws_connect(url)

        self.session=  ClientSession()
        self.ws=  await self.session.ws_connect(url)
                # self.on_open()
                # self.ws=ws
                # async for msg in ws:
                # while True:
        
        # self._status=1
    @property
    async def status(self):
        await self.receive()
        return self._status
        
    def count(self):
        # await asyncio.sleep(0.1)
        # while self._status == 0:
        #     await asyncio.sleep(0.5)
        print(len(self.message))
        print(self._status)
        return self._status
        
    async def create(self):
    #    await self.websocket_connect('ws://10.101.10.45/api/streams/id/bb30285a-7d11-484b-a8b0-4855cf8bf8ed')
        await self.websocket_connect('ws://10.101.10.45/api/streams/id/6f6ce77c-48a1-4043-8549-1eb72a1a45c7')
    #    asyncio.create_task(self.websocket_connect('ws://10.101.10.45/api/streams/id/bb30285a-7d11-484b-a8b0-4855cf8bf8ed'))
    
async def main():
    ws=WebsocketClient()
    task=await ws.create()
    # task= ws.websocket_connect('ws://10.101.10.45/api/streams/id/6f6ce77c-48a1-4043-8549-1eb72a1a45c7')
    # print(task)
    # await ws.count()
    # async for msg in ws:
    #     # print(msg)
    #     pass
    
    while (await ws.status == 0):
        # ws.count()
        pass
    #     # time.sleep(0.5)
    #     await asyncio.sleep(0.5)
    # task_list=[asyncio.create_task(ws.count())]
    # await asyncio.wait(task_list)
    # print(x[0].pop().result())
    ws.count()
if __name__ == '__main__':
    
    asyncio.run(main())


import json
from ..utils import request
from ..model.model import Model
from .DataManageModel import IESSimulationDataManageModel


class IESLabSimulation(object):
    def __init__(self, project={}):
        '''
            初始化
        '''
        self.id = project.get('id', None)
        self.name = project.get('name', None)
        self.__modelRid = project.get('model', None)
        self.project_group = project.get('project_group', None)
        if self.__modelRid is not None:
            self.model = Model.fetch(self.__modelRid)
        self.dataManageModel = IESSimulationDataManageModel(self.id)

    @staticmethod
    def fetch(simulationId):
        '''
            获取算例信息

            :params: simulationId string类型，代表数据项的算例id

            :return: IESLabSimulation
        '''
        try:
            r = request(
                'GET', 'api/ieslab-simulation/rest/simu/{0}/'.format(simulationId))
            project = json.loads(r.text)
            return IESLabSimulation(project)
        except:
            raise Exception('未查询到当前算例')

    def run(self, job=None, name=None):
        '''
            调用仿真 

            :params job:  调用仿真时使用的计算方案，不指定将使用算例保存时选中的计算方案
            :params name:  任务名称，为空时使用项目的参数方案名称和计算方案名称

            :return: 返回一个运行实例
        '''
        if job is None:
            currentJob = self.model.context['currentJob']
            job = self.model.jobs[currentJob]

        job['args']['simulationId'] = self.id
        return self.model.run(job, name=name)
    
    @staticmethod
    def createProjectGroup(group_name, desc=None, createById=None):
        '''
            创建项目组

            :params group_name: String 项目组名称 
            :params desc: String 项目组描述 可选参数
            :params createById Int 父项目组id  可选参数，如果是从已有项目组导入的项目组，必填此项

            :return: Int 返回创建的项目组id
        '''
        try:
            if createById is None: 
                isImport = 0
            else:
                isImport = 1
            payload = {
                'group_name': group_name,
                'desc': desc,
                'isImport': isImport,
                'createById': createById,
            }
            r = request(
                'POST', 'api/ieslab-simulation/rest/projectgroup/', data=json.dumps(payload))
            project = json.loads(r.text)
            return project.get('id', None)
        except Exception as e:
            raise Exception('创建项目组失败')

    @staticmethod 
    def createProject(name, project_group, desc=None, createById=None):
        '''
            创建项目

            :params name: String 项目名称 
            :params project_group: Int 父项目组id,
            :params desc: String 项目描述, 可选参数
            :params createById Int 父项目id, 可选参数, 如果是从已有项目导入的项目，必填此项

            :return: Int 返回创建的项目id
        '''
        try:
            if createById is None: 
                payload = {
                    'name': name,
                    'project_group': project_group,
                    'desc': desc
                }
            else:
                payload = {
                    'name': name,
                    'project_group': project_group,
                    'desc': desc,
                    'createById': createById
                }
            r = request(
                'POST', 'api/ieslab-simulation/rest/simu/', data=json.dumps(payload))
            project = json.loads(r.text)
            return project.get('id', None)
        except Exception as e:
            raise Exception('创建项目失败')




from aiohttp import ClientSession, WSMsgType
import asyncio
import time
# ws://10.101.10.45/api/streams/token/fe943635-0dc9-40c8-8a27-39d02af66894

    
class WebsocketClient():
    message = []
    _status=0
    # __init__(self):
    def on_close(self):
        self._status = 1
        print("close")
    def on_error(self,msg):
        self._status = 1
        print("error")
    def on_open(self):
        self._status = 0
        print("open")
    # websocket
    async def websocket_connect(self,url):
        
        # session = await ClientSession()
        # ws = await session.ws_connect(url)

        async with ClientSession() as session:
            async with session.ws_connect(url) as ws:
                self.on_open()
                # async for msg in ws:
                while True:
                    msg = await ws.receive()
                    if msg.type == WSMsgType.BINARY:
                        self.message.append(msg)
                    if msg.type == WSMsgType.TEXT:
                        self.message.append(msg)
                    elif msg.type == WSMsgType.CLOSED:
                        self.on_close()
                        break
                    elif msg.type == WSMsgType.ERROR:
                        self.on_error(msg.data)
                        break
        self._status=1
    @property
    async def status(self):
        await asyncio.sleep(0)
        return self._status
    def count(self):
        # await asyncio.sleep(0.1)
        # while self._status == 0:
        #     await asyncio.sleep(0.5)
        print(len(self.message))
        return self._status
        
    async def create(self):
       asyncio.create_task(self.websocket_connect('ws://10.101.10.45/api/streams/id/bb30285a-7d11-484b-a8b0-4855cf8bf8ed'))
       
async def main():
    ws=WebsocketClient()
    task=await ws.create()
    while (await ws.status == 0):
        pass
    ws.count()  
if __name__ == '__main__':
    
    asyncio.run(main())


import asyncio


async def module_a():
    print("start module_a")
    await asyncio.sleep(2) # 模拟 module_a 的io操作
    print('end module_a')
    return 'module_a 完成'

async def module_b():
    print("start module_b")
    await asyncio.sleep(1) # 模拟 module_a 的io操作
    print('end module_b')
    return 'module_b 完成'  

task_list = [
    module_a(),
 module_b(), 
]

done,pending = asyncio.run( asyncio.wait(task_list) )
for task in done:
    print(task.result())
print(done)


# coding=UTF-8

import json
import requests
import os
from collections import OrderedDict
from ..version import __version__
def request(method, uri, baseUrl=None, params={}, token=None, **kwargs):
    if baseUrl == None:
        baseUrl = os.environ.get('CLOUDPSS_API_URL', 'https://cloudpss.net/')
    url = requests.compat.urljoin(baseUrl,uri)
    token = os.environ.get('CLOUDPSS_TOKEN', None)
    if token:
        headers = {
            'Authorization': 'Bearer ' + token,
            'Content-Type': 'application/json; charset=utf-8'
        }
    else:
        raise Exception('token undefined')
    r = requests.request(method, url, params=params, headers=headers, **kwargs)

    if (uri.startswith('graphql')):
        if 'X-Cloudpss-Version' not in r.headers:
            raise Exception(
                '当前SDK版本（ver 3.X.X）与服务器版本（3.0.0 以下）不兼容，请更换服务器地址或更换SDK版本。')
        os.environ['X_CLOUDPSS_VERSION'] = r.headers['X-Cloudpss-Version']
        if float(r.headers['X-Cloudpss-Version']) < 4:
            raise Exception('当前SDK版本（ver '+__version__ +'）与服务器版本（ver ' +
                            r.headers['X-Cloudpss-Version'] +
                            '.X.X）不兼容，请更换服务器地址或更换SDK版本(pip 使用 pip install -U cloudpss 命令更新, conda 使用 conda update cloudpss 命令更新)。')

    if r.ok:
        return r
    if r.text =="":
        r.raise_for_status()
    if "statusCode" in r.text:
        t = json.loads(r.text)
        raise  Exception( str(t['statusCode']) + " "+t['message'])
    



from .job import Job
from ..utils import request
from .function import Function
from .functionExecution import FunctionExecution
import json
from deprecated import deprecated

__all__ = ['Function', 'Job', 'FunctionExecution']


def createJob():
    pass


@deprecated(
    version='3.0',
    reason="该类将在 5.0 版本移除，请使用 cloudpss.currentJob() 方法代替",
)
def currentJob():
    job = Job.current()
    job.message(
        "DeprecationWarning: Call to deprecated function (or staticmethod) cloudpss.function.currentJob(). (该类将在 5.0 版本移除，请使用 cloudpss.currentJob() 方法代替",
        level='warning')
    return job
    pass


def currentExecutor():
    """
        获取表示当前执行的 FunctionExecution 单例
    """
    return FunctionExecution.current()


def fetch(rid):
    """
        获取函数
    """
    query = """
        query ($rid:ResourceId!) {
                
            function(rid:$rid) {
                rid,
                documentation
                parameters
                implementType
                implement
                configs
                context
                name
                description
                type
                owner
                key
                executor
            }
        }
    """
    payload = {
        'query': query,
        'variables': {
            'rid': rid,
        }
    }
    r = request('POST', 'graphql', data=json.dumps(payload))
    data = json.loads(r.text)
    return Function(data['data']['function'])


def fetchMany(name=None, pageSize=10, pageOffset=0):
    pass



import json
from ..utils.httprequests import request


class IESLabEvaluationResult(object):
    _baseUri = 'api/ieslab-plan/taskmanager/'
    _kindNameMap = {
        "利润与利润分配": "getEconomyResult",
        "财务计划现金": "getFinancialPlanCashFlowResult",
        "资产负债": "getLiabilityAssetsResult",
        "投资使用计划与资金筹措": "getInvestPlanDataResult",
        "借款还本付息计划": "getLoanRepaymentPlanResult",
        "流动资金估算": "getFlowCashEvaluteResult",
        "资产折旧与摊销估算": "getFlowCashEvaluteResult",
        "总成本费用估算表": "getSumCostResult",
        "项目总投资现金流量": "getSumInvestFlowCashResult",
        "项目资本金现金流量": "getProjectCashFlowResult",
        "营业收入、税金、附加和增值税估算": "getIncomeTaxResult",
    }

    def __init__(self, simulationId, taskId=None, **keywords) -> None:
        """
            初始化
        """
        self.simulationId = simulationId
        self.timeId = keywords.get('timeId', 0)
        self.planId = keywords.get('planId', 0)
        self.cmdType = keywords.get('cmdType', None)

    def _fetchItemData(self, url, planID):
        '''
            获取planID对应的优化方案下resultType财务评估结果

            :param planID int 类型，表示优化方案的ID，数值位于0~优化方案数量之间
            :param resultType enum 类型，表示财务评价结果表格的类型

            :return: dict 类型，为源数据的引用，代表方案对应的财务评价基础参数信息
        '''
        r = request('GET',
                    url,
                    params={
                        "simu_id": self.simulationId,
                        "planId": planID,
                        "time": 0
                    })
        data = json.loads(r.text)
        return data

    def status(self):
        '''
            获取运行状态

            :return: boolean 类型
        '''
        # 定义一个字典，把命令类型映射到相应的方法
        cmd_dict = {
            None: self.GetOverviewResult,
            'energyEvaluation': self.GetEnergyEvaluationResult,
            'environmentalEvaluation': self.GetEnvironmentalEvaluationResult
        }
        # 从字典中获取对应的方法，如果没有找到，就抛出一个异常或返回False
        result = cmd_dict.get(self.cmdType, lambda x: None)(self.planId)
        if result is None or type(result) is list:
            return False
        return True


    def GetFinancialResult(self, resultType, planID):
        '''
            获取planID对应的优化方案下resultType财务评估结果

            :param planID int 类型，表示优化方案的ID，数值位于0~优化方案数量之间
            :param resultType enum 类型，表示财务评价结果表格的类型

            :return: dict 类型，为源数据的引用，代表方案对应的财务评价基础参数信息

        '''
        assert (resultType in self._kindNameMap), "数据类型不存在"
        kind = self._kindNameMap.get(resultType, resultType)
        url = self._baseUri + kind
        list = self._fetchItemData(url, planID)
        dict_result = dict()
        for val in list['results']:
            for k, v in val.items():
                dict_result[k] = v
        return dict_result['data']

    def GetOverviewResult(self, planID):
        '''
            获取当前结果类对应的优化方案下的概览结果
            :param planID int 类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: array类型，代表该方案对应的概览结果
        '''
        r = request('GET',
                    "api/ieslab-plan/taskmanager/getOverviewResult",
                    params={
                        "simu_id": self.simulationId,
                        "planId": planID,
                        "time": self.timeId
                    })
        result = json.loads(r.text)
        if len(result['results']) > 0 and len(result['results'][0]['data']) > 0:
            return result['results'][0]['data']
        return []

    def GetEnergyEvaluationResult(self, planID):
        '''
            获取当前结果类对应的优化方案下的能效评价
            :param planID int 类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: array类型，代表该方案对应的能效评价结果
        '''
        r = request('GET',
                    "api/ieslab-plan/taskmanager/getEnergyEvaluation",
                    params={
                        "simu_id": self.simulationId,
                        "planId": planID,
                        "time": self.timeId
                    })
        result = json.loads(r.text)
        if len(result['results']) > 0 and len(result['results'][0]['data']) > 0:
            return result['results'][0]['data']
        return []

    def GetEnvironmentalEvaluationResult(self, planID):
        '''
            获取当前结果类对应的优化方案下的环保评价
            :param planID int 类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: array类型，代表该方案对应的环保评价结果
        '''
        r = request('GET',
                    "api/ieslab-plan/taskmanager/getEnvironmentalEvaluation",
                    params={
                        "simu_id": self.simulationId,
                        "planId": planID,
                        "time": self.timeId
                    })
        result = json.loads(r.text)
        if len(result['results']) > 0 and len(result['results'][0]['data']) > 0:
            return result['results'][0]['data']
        return []


import os, sys

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))
import cloudpss
import time
tk = 'eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MTAyNiwidXNlcm5hbWUiOiJncmV5ZCIsInNjb3BlcyI6W10sInR5cGUiOiJhcHBseSIsImV4cCI6MTcxNDI2ODg5MSwiaWF0IjoxNjgzMTY0ODkxfQ.PdMMmsX1etXdPyS6ktZB3LvgRvqh6FL5jrJj2dfNhQyMKktWrKo64JLegWqsqnEJG-zwyTRS5vVsg-6eiVinXQ'
apiURL = 'https://cloudpss.net/';
username = 'greyd'
projectKey = '3m9btest'
cloudpss.setToken(tk)
os.environ['CLOUDPSS_API_URL'] = apiURL
project = cloudpss.Model.fetch('model/'+username+'/'+projectKey)
job = project.getModelJob("潮流计算方案 1")[0]
config = project.getModelConfig("参数方案 1")[0]
runner=project.run(job=job,config=config)

project.getComponentsByRid


while runner.status()<1:
    for log in runner.result.getLogs():
        print(log)
        if('data' in log.keys() and 'content' in log['data'].keys()):
            print(log['data']['content'])
    print(runner.status())
    if runner.status()==-1:
        break
    time.sleep(2)
try:
    print(runner.result.getBuses())
except:
    print('nothing')

